package com.ether;

import com.ether.annotation.ControllerMethod;
import com.ether.annotation.Mandatory;
import com.ether.annotation.ParamName;
import com.medias.database.objects.Parametre;
import com.medias.database.objects.Plateforme;
import net.sf.json.JSONObject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author vmipsl
 * @date 17 feb 2011
 */
public class Controller
        extends ControllerEther
{
    /** *********************************************************** **/
    /** *********************** VIEWS ***************************** **/
    /** *********************************************************** **/
    // Default view if methodName is unknown
    public ModelAndView home( final HttpServletRequest request, final HttpServletResponse response )
            throws WebException
    {
        return new ModelAndView( "index" );
    }

    @ControllerMethod(view = VIEW_VISUALIZATION)
    public Map<String, Object> view()
            throws ServiceException
    {
        return new HashMap<String, Object>();
    }

    @ControllerMethod(view = VIEW_VISUALIZATION_PARAMETER_BY_PLATEFORM)
    public Map<String, Object> viewParametersByPlateform()
            throws ServiceException
    {
        final List<Plateforme> plateforms = _etherService.getAllPlateforms();

        final Map<String, Object> model = new HashMap<String, Object>();
        model.put( "plateforms", getJsonHelper().toJSON( plateforms ) );
        return model;
    }

    /** *********************************************************** **/
    /** *********************** CALLS ***************************** **/
    /** ********************************************************** **/
    @ControllerMethod(jsonResult = true)
    public JSONObject searchParametersByPlateform( @Mandatory @ParamName(ParameterConstants.PARAMETER_ID) final Integer plateformId )
            throws ServiceException, WebException
    {
        final List<Parametre> parametersByPlateform = _etherService.getParametersByPlateformId( plateformId );

        final JSONObject result = new JSONObject();
        result.put( ParameterConstants.PARAMETER_PARAMETERS, getJsonHelper().toJSON( parametersByPlateform ) );
        return result;
    }

    @Required
    public void setEtherService( @NotNull final EtherService etherService )
    {
        _etherService = etherService;
    }

    private static final Log LOGGER = LogFactory.getLog( Controller.class );

    private static final String VIEW_VISUALIZATION = "visualization/visu";
    private static final String VIEW_VISUALIZATION_PARAMETER_BY_PLATEFORM = "visualization/visu_parameter_by_pf";

    private EtherService _etherService;
}