package com.ether;

import com.medias.Context;
import gov.noaa.pmel.sgt.dm.SGTMetaData;
import gov.noaa.pmel.sgt.dm.SimpleLine;
import gov.noaa.pmel.util.GeoDateArray;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.springframework.context.ApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import javax.imageio.ImageIO;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.text.ParseException;
import java.util.Date;
import java.util.List;

/**
 * @author vmipsl
 * @date 17 june 2011
 */
public class ControllerPlot
        extends HttpServlet
{
    public void init( final ServletConfig servletConfig )
            throws ServletException
    {
        try
        {
            final ApplicationContext appContext = WebApplicationContextUtils.getRequiredWebApplicationContext( servletConfig.getServletContext() );
            _etherService = (EtherService) appContext.getBean( "etherService", EtherService.class );
            _etherPlotService = (EtherPlotService) appContext.getBean( "etherPlotService", EtherPlotService.class );
        }
        catch( Throwable tx )
        {
            LOGGER.error( "Error initializing EtherService.", tx );
            throw new ServletException( "Error initializing EtherService.", tx );
        }
    }

    /**
     * Creates an image with the plot and send it to the response
     * Call by a jsp with <img src="visualization/plotEther?....">
     *
     * @param request
     * @param response
     * @throws ServletException
     */
    public void doGet( final HttpServletRequest request, final HttpServletResponse response )
            throws ServletException
    {
        try
        {
            final Integer plateformId = Integer.valueOf( request.getParameter( "plateformId" ) );
            final Integer parameterId = Integer.valueOf( request.getParameter( "parameterId" ) );
            final String dateBegin = request.getParameter( "dateBegin" );
            final String dateEnd = request.getParameter( "dateEnd" );
            final String pofBegin = "2009-07-13 13:00";
            final String pofEnd = "2009-07-14 14:00";
            final Date formatedDateBegin = DateHelper.parseDate( pofBegin, DateHelper.ENGLISH_DATE_PATTERN );
            final Date formatedDateEnd = DateHelper.parseDate( pofEnd, DateHelper.ENGLISH_DATE_PATTERN );

            //** ******************************************************************** **//
            // TODO : replace List<Data> by List<value> and List<double>
            //** ******************************************************************** **//
            final List<Pair<Double, Date>> values = _etherService.getValuesByPlateformByParameterByPeriod( plateformId, parameterId, formatedDateBegin, formatedDateEnd );
            final double[] dataArray = extractDoubles( values );
            final Date[] dateValues = extractDates( values );

            final GeoDateArray dateArray = new GeoDateArray( dateValues );
            //** ******************************************************************** **//
            //** ******************************************************************** **//


            final SimpleLine data = new SimpleLine( dateArray, dataArray, "legend" );
            SGTMetaData meta = new SGTMetaData( "Longitude", "degrees East", false, false );
            data.setXMetaData( meta );

            meta = new SGTMetaData( "parameterName", "parameterUnit", false, false );
            data.setYMetaData( meta );

            final MegapoliPlot megapoliPlot = new MegapoliPlot();
            megapoliPlot.setTitle( "Mobilis LIDAR" );
            megapoliPlot.setData( data );

            final BufferedImage bufferedImage = _etherPlotService.createJPane( megapoliPlot, Context.getLocale( request ) );

            ImageIO.write( bufferedImage, "png", response.getOutputStream() );
        }
        catch( IOException e )
        {
            throw new ServletException( "Error : no possibity to write image in response", e );
        }
        catch( ServiceException e )
        {
            throw new ServletException( "Error : no possibility to extract data from base", e );
        }
        catch( ParseException e )
        {
            throw new ServletException( "Error : invalid dates, no parsing available", e );
        }
    }

    // TODO : remove this code !! ultracrados !
    private double[] extractDoubles( @NotNull final List<Pair<Double, Date>> datas )
    {
        final double[] doubles = new double[datas.size()];
        int i = 0;
        for( final Pair<Double, Date> data : datas )
        {
            doubles[i] = data.getFirstValue().doubleValue();
            i++;
        }
        return doubles;
    }

    // TODO : remove this code !! ultracrados !
    private Date[] extractDates( @NotNull final List<Pair<Double, Date>> datas )
    {
        final Date[] dates = new Date[datas.size()];
        int i = 0;
        for( final Pair<Double, Date> data : datas )
        {
            dates[i] = data.getSecondValue();
            i++;
        }
        return dates;
    }

    private static final Log LOGGER = LogFactory.getLog( ControllerPlot.class );

    // Dimensions of the jPanes
    private static final int MAIN_WIDTH = 1000;
    private static final int MAIN_HEIGHT = 1000;

    private EtherService _etherService;
    private EtherPlotService _etherPlotService;

}